package libgajadiary;

$libname = "libgajadiary";
$update = "20040212";

use libgajaf;
use Time::Local;
require "gajadiary.conf";
&init_gajadiary();

sub init_gajadiary{
    &defelement("article","p");
    &defelement("div","p");
    &defelement("p");
    &defelement("dl","dd");
    &defelement("dt");
    &defelement("dd");
    &defelement("ul","li");
    &defelement("ol","li");
    &defelement("li");
    &defelement("table","");
    &defelement("pre","");
    &defelement("blockquote","p");

    $PATTERN{URL} = '(?:|https?|ftp)://[\w;/?:@&=\+\$\.,\-!~\*\%\'\(\)\#]+';

    @STATE = ("article");

    %MONTH      = ("Jan",1,"Feb",2,"Mar",3,"Apr",4,"May",5,"Jun",6,
		   "Jul",7,"Aug",8,"Sep",9,"Oct",10,"Nov",11,"Dec",12);
    $MONTH      = "(?:".join('|',keys %MONTH).")";

    my $NOW = time();
    ($second,$minute,$hour,$day,$month,$year
	  ,$wday,$wyear,$stime,$part_of_month)
	= mylocaltime($NOW);
    $logfile = time2logfile($NOW);
    $today = sprintf("%d.%d",$month,$day);
    $pointer = sprintf("%02d%02d%02d",$day,$hour,$minute);
    
    if(not defined @day){
	@day = ("Sunday","Monday","Tuesday","Wednesday",
		"Thursday","Friday","Saturday");
    }
    if(not defined @month){
	@month = ("","Jan","Feb","Mar","Apr","May","Jun",
		  "Jul","Aug","Sep","Oct","Nov","Dec");
    }
    if(not defined @unit){
	@unit    = ("a","b");
    }

    $modifiedMes = 
	    sprintf("%s, %02d-%s-%02d %02s:%02s:%02s JST"
		    ,$day[$wday],$day,$month[$month]
		    ,$year,$hour,$minute,$second);
    $modifiedMes = "<!--Last-Modified:$modifiedMes-->";

    $Commit_OK = 1;

    if(!(-e "$basedir/$logfile")){
	$oldlogfile = time2logfile($NOW - $days_in_log * 86400);
	$chLogFlag = 1;
	&setupNewLog();
    }
}

sub setCommit{
    $Commit_OK = $_[0];
}

sub defelement{
    local($tag,$inline) = @_;
    if(!defined($inline)){
	$elemlist{$tag} = 2;
	$pbegin{$tag} = "<$tag>";
	$pend{$tag}   = "</$tag>";
    }elsif($inline eq ""){
	$elemlist{$tag} = 1;
	$pbegin{$tag} = "";
	$pend{$tag}   = "";
    }else{
	$elemlist{$tag} = 1;
	$pbegin{$tag} = "<$inline>";
	$pend{$tag}   = "</$inline>";
    }
}

sub setupNewLog{
    local($temp);
    open(TEMPLATE,"$basedir/$templatefile");
    local($unitface)   = $unit[int(($day-1) / $days_in_log)];
    local($JPunitface) = $JPunit[int(($day-1) / $days_in_log)];
    local($partname) = $logfile; $partname =~ s/\.html//;
    &jcode::convert(*unitface,"$filecoding");
    while(<TEMPLATE>){
	&jcode::convert(*_,"$filecoding");
	s/<!-- Here Embedded BASE:href-->/<base href="$basehttpdir$logfile">/g;
	s/<!--Year-->/$year+2000/eg;
	s/<!--Month-->/$month/g;
	s/<!--FileURL-->/$basehttpdir$logfile/g;
	s/<!--Partname-->/$partname/g;
	s/<!--Unit-->/$unitface/g;
	$temp = $temp.$_;
    }close(TEMPLATE);
    open(LOG,">$basedir/$logfile");
    print LOG $temp;
    close(LOG);
}

sub readAliasFile{
    open(ALIAS,"$basedir/$aliasfile");
    while(<ALIAS>){
	&jcode::convert(*_,"euc");
	/(.*?)\t+(.*)/;
	my($alias,$entity) = ($1,$2);
	$Alias{$alias} = "$entity";
    }
    close(ALIAS);
}

sub addAlias{
    my($alias,$entity) = @_;
    $Alias{$alias} = "$entity";
    if($Commit_OK){
	open(ALIAS,">> $basedir/$aliasfile");
	print ALIAS "$alias\t$entity\n";
	close(ALIAS);
    }
}

sub readNewArticle{
    open(NEWARTICLE,"$basedir/$newarticle");
    while(<NEWARTICLE>){
	$source .= $_;
    }
    
    @output = &convertToHTML($source);
}

sub showNewArticle{
    my($outcode) = @_;
    my($output) = "<hr />\n";
    foreach(@output){
	local($url,$text) = @$_;
	$output = $text.$output;
    }
    &jcode::convert(\$output,$outcode);
    print $output;
}

sub convertToHTML{
    local($_) = @_;
    local($section,$articlenum) = (0,0);
    local($title,$source,$secname,$articleurl,$i,$pref);
    local(@_,@tmp,@output);
    local($output,$url);

    &jcode::convert(*_,"euc");
    &parsePRE(*_);
    
    $_ = format_list($_);
    split(/\n/,$_);

    $i = 0;
    
    foreach(@_){
	if(s/\\$//){
	    $pref .= $_."\n";
	}else{
	    push(@tmp,"$pref$_");
	    $pref = "";
	}
    }@_ = @tmp;undef(@tmp);

    foreach(@_){
	if(/^$/){next;}
	if(s/^\#\#//){
	    my($topic) = $_;
	    $title = $topic;
	    my($topic_key);
	    if($articlenum != 0){
		my $log = $logfile;
		$log =~ s/\.html//;
		push(@tmp,"<p class='advance_ptr'><a href='article.cgi?$log$articleptr'>+comment(0)</a></p>\n</div>");
	    }
	    $articleptr  = $pointer.int2b52($articlenum++);
	    if($topic =~ /^\[(.*?)\]/){
		open(TOPICINDEX,">> $basedir/$topictypeindex");
		print TOPICINDEX "$1,$',$logfile#$articleptr\n";
		close(TOPICINDEX);
		$topic = "$'";
	    }
	    $section = 0;
	    push(@tmp,"<!gajadiaryarticle />$articleptr");
	    push(@tmp,"<!gajadiarypointer $articleptr />");
	    push(@tmp,"<div class='Article'>");
	    push(@tmp,
		 "<h4>".
		 "<a href='#$articleptr' name='$articleptr'>@</a>");
	    @tmp[$#tmp] .= $topic if($topic ne "");
	    @tmp[$#tmp] .= "</h4>";
	}elsif(s/^\#//){
	    $secptr = $articleptr.$section;
	    push(@tmp,"<!gajadiarypointer $secptr />");
	    push(@tmp,
		 "<p class='newsection'>".
		 "<a href='#$secptr' name='$secptr'>_</a>".
		 $_."</p>");
	    $section++;
	}else{
	    s/^\\\#/\#/;
	    push(@tmp,$_);
	}
    }@_ = @tmp;undef(@tmp);
    
    while(&resolveAlias(@_) == 1){}
    
    foreach(@_){
	if(m%^<!gajadiarypointer (.*?) />%){$secname = $1;next;}

	# ignore comment-outed lines
	if(/^<!--/ && !/-->/){}
	elsif(/^-->$/){}
	elsif(/^ *<(.*?)[> ]/){	# for line begin by <tag>.
	    $tag = $1;
	    if(lc($tag) eq "div"){s/^<div>/<div class='subcolumn'>/i;}
	    if($tag =~ /^a/i){
		$_ = $pbegin{$STATE[0]}.$_.$pend{$STATE[0]};
	    }
	    if($tag =~ /^\/(.*)/ &&
	       $elemlist{lc($1)} == 1 &&
	       $STATE[0] eq lc($1)       ){
		shift(@STATE);
	    }
	    elsif($elemlist{lc($tag)} == 1){
		unshift(@STATE,lc($1));
	    }elsif($elemlist{lc($tag)} == 2 && !(m%</$tag>%)){
		$_ .= "<\/".$tag.">";
	    }
	    if(s/^<dt>//){
		$_ = "<dt>"
		    ."<a href='#$pointer-$section' name='$pointer-$section'>"
			."_</a>".$_;
		$section++;
	    }
	}else{ # other lines
	    $_ = $pbegin{$STATE[0]}.$_.$pend{$STATE[0]};
	}

	$output .= $_."\n";
    }
    my $log = $logfile;
    $log =~ s/\.html//;
    $output .= "<p class='advance_ptr'><a href='article.cgi?$log$articleptr'>+comment(0)</a></p>\n</div>\n";

    $url = "$logfile#$pointer";

    &jcode::convert(*output,"$filecoding");
    
    @_ = split(m%<!gajadiaryarticle />%,$output);

    undef(@output);
    foreach(@_){
	s/^(.*?)\n//;
	if($1 ne ""){
	    my $ptr = $1;
	    &parseNOTE($ptr,\$_);
	    $_ =~ /<h4>.*?<\/a>(.*?)<\/h4>/;
	    my $title = $1;
	    push(@output,[$ptr,$_,$title]);
	}
    }
    return @output;
}

sub resolveAlias{
    my($state) = 0;
    my($warn);
    my $secname;
    grep{
	if(m%^<!gajadiarypointer (.*?) />%){$secname = $1;}
	my(@token) = split(/(\\\@|\@{.*?}|{.*?}|\@\[.*?\])/,$_);
	my $result;
	while(@token){
	    $_ = shift(@token);
	    shift(@token) while(@token && !$token[0]);
	    if(/\\\@/){
		$result .= '@';
	    }elsif(/^\@{(.*?)}$/){
		my $key = $1;
		if($token[0] =~ /^{(.*?)}$/){
		    # add token;
		    shift(@token);
		    my $entity = $1;
		    print "KEY $key / ENTITY $entity\n";
		    if(exists $Alias{$key}){
			print STDERR "warning: Alias $key already exists : [$Alias{$key}].\n";
			$warn = 1;
		    }else{
			$entity = "<a href='$entity'>$key</a>"
			    if($entity =~ /^$PATTERN{URL}$/);
			addAlias($key,$entity);
			print STDERR "add new Alias [$key] as [$entity]\n";
		    }
		}
		$result .= $Alias{$key} || $key;
	    }else{
		$result .= $_;
	    }
#	    print "  result:$result\n";
	}
#	print "$result\n";
	$_ = $result;
    }@_;
    if($warn == 1){
	print STDERR "abort.\n";
	exit(1);
    }
    return $state;
}

sub parsePRE{
    local(*text) = @_;
    $text =~ s{<pre>(.*?)</pre>}{"<pre>".&parsePREsub($1)."</pre>"}seg;
    return;
}

sub parsePREsub{
    my($text) = @_;
    $text =~ s/\&/\&amp;/g;
    $text =~ s/</&lt;/g;
    $text =~ s/>/&gt;/g;
    $text =~ s{^\#}{\\#}mg;
    return $text;
}

sub parseNOTE{
    local($basename,*text) = @_;
    local($note);
    local($NOTEcnt,$tmp) = (0,0);
    ($text,$note) = &parseNOTEsub($basename,$text,0);
    if($NOTEcnt != 0){
	$text .= "<hr class='divnote'>\n<div class='note'>\n";
	$text .= $note."</div>\n";
    }
}

sub parseNOTEsub{
    local($basename,$_,$here) = @_;
    local($note);
    if(/<!--NOTE:/){
	while(s/<!--NOTE:(.*)//s){
	    $NOTEcnt++;
	    $after = $1;
	    local($pos,$end) = (0,0);
	    do{
		$pos = index($after,"<!--",$pos+1);
		$end = index($after,"-->",$end+1);
		if($end < $[){
		    print STDERR "$libname Error :".
			"Missing NOTE's end mark('-->')\n";
		    exit(0);
		}
	    }while(($pos >= $[) && ($pos < $end));
	    $_ .= "<a href='#".$basename."N".$NOTEcnt."'>".
		"(*".$NOTEcnt.")</a>";
	    $_ .= substr($after,$end+3);
	    $note .= "<p><a name='".$basename."N".$NOTEcnt."'>(*".$NOTEcnt.")</a>";
	    $note .= substr($after,0,$end)."</p>\n";
	}
	($note,$tmp) = &parseNOTEsub($basename,$note,$here+1);
	$note .= $tmp;
    }
    return ($_,$note);
}

sub NonNumberTipsCount{
    local($i) = $NonNumberTipsCount++;
    local($_) = "";
    do{
	$_ .= substr("ABCDEFGHIJKLMNOPQRSTUVWXYZ",($i % 26),1);
	$i = int($i / 26);
    }while($i!=0);
    return $_;
}

sub renewSignature{
    foreach(@output){
	my($anchor,$text,$title) = @$_;
	print "adding Signature of $anchor...\n";
	&libgajaf::addSignature("$title\0$logfile#$anchor",$text);
    }
}

sub renewDigest{
    local($before,$after,$article);
    local(@article,$output);
    print "Renew Digest file ...\n";
    open(DIGEST,"$basedir/$digestfile");
    while(<DIGEST>){
	if(/^<!--Last-Modified:.*/){
	    /(?:Sunday|Monday|Tuesday|Wednesday|Thursday|Friday|Saturday)\s*,\s*(\d{2})-($MONTH)-(\d{2,4}) (\d{2}):(\d{2}):(\d{2}) ([^ \)\n\-]+|[+-]\d{4})/;
	    local($lmday) = sprintf("%d.%d",$MONTH{$2},$1);
	    if($lmday eq $today){
		print "same day renew...\n";
		$samedayRenew = 1;
	    }
	    $_ = "$modifiedMes\n";
	}
	if($chLogFlag == 1){
	    if(/<base href=(["''"])(.*?)\1(.*?)>/i){
		$oldlogfile = $2;
		print "Change BASE:$oldlogfile to $logfile\n";
		$oldlogfile =~ s/$basehttpdir//;
		s/<base href=(['""']).*?\1/<base href='$basehttpdir$logfile'/i;
	    }
	}
	$before .= $_;
	if(/<div class=(['""']?)freetalkMain\1>/){
	    if($samedayRenew == 1){$_ = <DIGEST>;$before .= $_;}
	    last;
	}
    }
    while(<DIGEST>){
	if(/<div class=(['""']?)Article\1>/i){
	    push(@article,$article);$article = "";
	}elsif(/<\/div><!-- end of FreetalkMain -->/){
	    push(@article,$article);$article = "";
	    last;
	}
	if($chLogFlag == 1){s/href=(['""'])\#/href=$1$oldlogfile\#/ig;}
	$article .= $_;
    }
    $after .= $_;
    while(<DIGEST>){
	$after .= $_;
    }
    close(DIGEST);
    
    open(DIGEST,"> $basedir/$digestfile");
    print DIGEST $before;
    print DIGEST "<h3>$today</h3>\n" if($samedayRenew != 1);
    foreach(reverse @output){
	my($url,$text) = @$_;
	print DIGEST $text;
    }
    print DIGEST "<hr>\n" if($samedayRenew != 1);
    splice(@article,-(scalar(@output)));
    foreach(@article){
	print DIGEST $_;
    }
    print DIGEST $after;
    close(DIGEST);
}

sub renewLog{
    local($before,$after);
    local($output);
    foreach(@output){
	local($url,$text) = @$_;
	$output .= $text;
    }
    print "Renew Log file ...\n";
    open(LOG,"$basedir/$logfile");
    while(<LOG>){
	if(/^<!--Last-Modified:.*/i){
	    /(?:Sunday|Monday|Tuesday|Wednesday|Thursday|Friday|Saturday)\s*,\s*(\d{2})-($MONTH)-(\d{2,4}) (\d{2}):(\d{2}):(\d{2}) ([^ \)\n\-]+|[+-]\d{4})/;
	    local($lmday) = sprintf("%d.%d",$MONTH{$2},$1);
	    if($lmday eq $today){
		print "same day renew...\n";
		$samedayRenew = 1;
	    }
	    $_ = "$modifiedMes\n";
	}
	if(/<\/div><!-- end of FreetalkMain -->/i){$after=$_;last;}
	$before = "$before$_";
    }
    while(<LOG>){$after = "$after$_"}
    close(LOG);

    open(LOG,">$basedir/$logfile");
    print LOG $before;
    if($samedayRenew != 1){
	print LOG "<hr>\n";
	print LOG "<h3>$today</h3>\n";
    }
    print LOG $output;
    print LOG $after;
    close(LOG);
}

sub URLENCODE{
    local($_) = @_;
    local($ret);
    while(s/(.)//){
	local($c) = $1;;
	if(!($c =~ /[a-zA-Z0-9\.]/)){
	    $c = "%".unpack("H2",$c);
	}
	$ret .= $c;
    }
    return $ret;
}

sub ISBN{
    local($_) = @_;
    print "ISBN called :$_\n";
    ($book,$isbn) = split(/,/);
    return "<a href='$isbn'>$book</a>";
}

sub regarg{
    local($_) = @_;
    s/\([^\'"](.*?)[\'"]\)/\("$1"\)/g;
    return $_;
}

sub int2b52{
    local($_) = @_;
    local($mod,$ret);

    do{
	$mod = $_ % 52;
	$_   = int($_ / 52);
	$ret = substr("abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ"
		      ,$mod,1).$ret;
    }while($_ != 0);
    return $ret;
}

sub eval_without_pre{
    my($func,$text) = @_;
    my $result;
    foreach(split(m{(<pre>.*?</pre>)}s,$text)){
	if(/^<pre>/){
	    $result .= $_;
	}else{
	    $result .= &$func($_);
	}
    }
    return $result;
}

sub time2logfile{
    my $time = shift;
    $time ||= time();

    return sprintf("%02d%02d%s.html",(mylocaltime($time))[5,4,9]);
}

sub logfile2time{
    my $logfile = shift;
    $logfile =~ /(\d{2})(\d{2})(.)/;
    my($y,$m,$u) = ($1,$2,$3);
    my($d) = 1 + $days_in_log * index("abcdefghij",$u);
    return timelocal(0,0,0,$d,$m - 1,$y+100);
}

sub prevlogfile{
    my($basename) = shift;
    my $time = logfile2time($basename);
    my $name = time2logfile($time);
    while($name eq $basename){
	$time -= 24 * 60 * 60;
	$name = time2logfile($time);
    }
    return $name;
}

sub mylocaltime{
    $time = shift;
    $time ||= time();
    my($second,$minute,$hour,$day,$month,$year,$wday,$wyear,$stime)
	= localtime($time);
    $month++;
    while($year >= 100){$year -= 100;}
    my($part_of_month) =
	substr("abcdefghij",int(($day<31 ? $day-1 : $day-2) / $days_in_log),1);
    return ($second,$minute,$hour,$day,$month,$year
	    ,$wday,$wyear,$stime,$part_of_month);
}

sub format_list{
    return eval_without_pre(\&format_list_core,@_);
}

sub format_list_core{
    my($text) = @_;

    my($result);
    my(@level);
    my($mode) = 0;
    foreach(split(/\n/,$text)){
	/^((?:\s|\xA1\xA1)*)(.*?)$/;
	my $level     = length($1);
	my $indent    = ' ' x $level;
	my $content   = $2;
	my $list_mark = 0;
	do{$level++; 
	   $list_mark++;} if($content =~ s/^\*/<li>/);
	$list_mark++ if($content =~ s/^<li>\#/<li>/);
	
	my $current = $level[0];
	if($level < $current->{level}){
	    $result .= format_list_unshift_level($level,\@level,\$mode);
	    $result .= '</li>' if($mode && $list_mark);
	}elsif($level == $current->{level}){
	    $result .= '</li>' if($mode && $list_mark);
	}else{# if($indent > $current->{level});
	    if($list_mark){
		my $type = ('','ul','ol')[$list_mark];
		unshift(@level,{ level => $level,
				 type  => $type       });
		$result .= ' ' x ($level - 1) . "<$type>\n";
	    }
	}
	$result .= $indent.$content."\n";
	$mode |= $list_mark;
	$mode = 0 if($level[0] eq '');
    }
    $result .= format_list_unshift_level(0,\@level,\$mode);
    $result =~ s{\n(\s*)</li>}{</li>\n$1}g;
    $result =~ s{</li>(\n\s*)<ul>}{$1<ul>}sg;
    return $result;
}

sub format_list_unshift_level{
    my($indent,$level,$mode) = @_;
    my($result,$lv,$ty);
    while(@$level){
	$lv = $$level[0]->{level};
	$ty = $$level[0]->{type};
	last unless ($indent < $lv);
	$result .= ' ' x ($lv - 1) . "</li></$ty>\n" if($$mode);
	shift(@$level);
    }
    $mode = 0 if($lv eq '');
    return $result;
}
1;
